# $Id: record.py 3738 2010-06-09 11:16:46Z place $
# $URL: https://svn.non-gnu.uvt.nl/uvt-dev/tags/sources/meresco/bzv-9.1.0/src/record.py $
from bzv.simplejson.encoder import JSONEncoder
from cgi import parse_qs
from cqlparser.cqlparser import parseString as parseCQL
from lxml.etree import parse, tostring
from merescocore.components.http import utils as httputils
from merescocore.framework import Observable
from storage.hierarchicalstorage import HierarchicalStorageError
from traceback import format_exc
from urlparse import urlsplit
from xml.sax.saxutils import escape
from urllib import urlopen

def okJSON():
    CRLF = "\r\n"
    ContentTypeJSON = "Content-Type: application/json"
    Ok = "HTTP/1.0 200 Ok" + CRLF
    okJSON = "HTTP/1.0 200 OK" + CRLF + \
             ContentTypeJSON + CRLF + \
             CRLF
    return okJSON

def forbiddenXml():
    return "HTTP/1.0 403 Forbidden" + httputils.CRLF + \
        httputils.ContentTypeXml  + httputils.CRLF + \
        httputils.CRLF

def add(aDictionary, key, value):
    if value:
            aDictionary[key] = value

def makeUrlContent(url):
    s = '<a target="_blank" href="%(url)s">%(url)s</a>' %{'url': escape(url)}
    return s

def makeDoiContent(doi):
    s = '<a target="_blank" href="http://dx.doi.org/%(doi)s">%(doi)s</a>' %{'doi': doi}
    return s

def makePermaLink(id):
    s = '<a target="_blank" href="/id/%(id)s">%(id)s</a>' %{'id': escape(id)}
    return s

class Record(Observable):
    def __init__(self, infra = None):
         Observable.__init__(self)
         self.infra = infra
         self.infra.logging.debug('Starting record logging')
         self.logger = infra.logging.getLogger('meresco.record')
         self.logger.debug('Starting record logger')
         self.serverConfig  = self.infra.configRoot
         self.nsMap = self.serverConfig.getSectionDict('namespaces')
         self._recordId = ""

    def xpath(self, node, path):
        #FIXME: what if node == None
        result = node.xpath(path, self.nsMap)
        return result and result[0] or ''

    def nodeset(self, node, path):
        #FIXME: what if node == None
        return node.xpath(path, self.nsMap)

    def createRecord(self, recordId, format = "full"):
        if recordId != self._recordId:
            if not self.getMetadata(recordId): return {}

        if format == "full":
            self.makeDictionary()
        elif format == "short":
            self.makeShortDict()
        elif format == "citec":
            self.makeCitecDict()
        elif format == "availability":
        #FIXME niet geimplementeerd
            self.makeAvailDict()
        return self.record

    def getMetadata(self, recordId):
        baseurl = self.serverConfig.get('url', section='dbserver')
        recId = recordId
        # FIXME er is een probleem met ids die haakjes bevatten
        #recId = recordId.replace('(', '\(').replace(')', '\)')
        #recId = recordId.split(':', 1)
        #recId = recId[0] + ':"' + recId[1] + '"'
        self.logger.debug(recId)
        try:
            if recordId.split(':')[0] in ['opc-uvt-nl']:
                try:
                    _metadata = parse(self.any.getStream(recId, 'mods'))
                    _meta     = parse(self.any.getStream(recId, 'meta'))
                except HierarchicalStorageError:
                    url = baseurl + \
                            "/sru?version=1.1&operation=searchRetrieve&recordSchema=mods&x-recordSchema=meta&query=meta.upload.id+exact+\"" + \
                            recordId + "\""
                    _metadata = parse(urlopen(url))
                    _meta = _metadata
            else:
                try:
                    _metadata = parse(self.any.getStream(recId, 'metadata'))
                    _meta     = parse(self.any.getStream(recId, 'meta'))
                except HierarchicalStorageError:
                    url = baseurl + "/sru?version=1.1&operation=searchRetrieve&x-recordSchema=meta&query=meta.upload.id+exact+\"" + recordId + "\""
                    self.logger.debug("Cannot retrieve record; remote host will be tried, url +'" + url +'"')
                    _metadata = parse(urlopen(url))
                    _meta = _metadata
#            self.infra.logging.debug(self.infra.prpt("----METADATA---",self._metadata))

            self._parts      = self.nodeset(_meta, '//meta:parts/meta:part')
            self._recordId   = recordId
            self._collection = self.xpath(_meta, '//meta:meta/meta:repository/meta:collection/text()')
            self._db         = self.xpath(_meta, '/meta:meta/meta:repository/meta:repositoryGroupId/text()')
            self._metadata = _metadata

            try:
                self._setGenre()
            except Exception:
                self._genre=''
                self.logger.error("no Genre for " + self._recordId)

        except Exception:
            self.logger.error("No metadata for record: " + str(recordId))
            self.logger.error(format_exc())
            return False

        return True

    def getSid(self, recordId):
        if recordId != self._recordId: self.getMetadata(recordId)
        return self._collection + ':' + self._db

    def getOrder(self):
###### Note that all fields must be listed here
######      otherwise they will not be displayed.
###### This is a public method.
       order = {
           1: 'Title',
           3: 'Author(s)',
           4: 'Collaborator(s)',
           5: 'Editor(s)',
           6: 'Supervisor(s)',
           7: 'Corporation',
           8: 'Source',
           9: 'Publication date',
           10: 'URL(s)',
           11: 'Doi',
           12: 'Dataset',
           13: 'Abstract',
           14: 'Person as subject',
           15: 'Keyword(s)',
           16: 'Classification',
           17: 'Generated JEL',
           18: 'ISSN',
           19: 'ISBN',
           20: 'Publisher',
           21: 'Extent',
           22: 'Series',
           23: 'Note(s)',
           24: 'Reference',
           25: 'Publication type',
           26: 'Location',
           27: 'Related Titles',
           28: 'Cited by',
           29: 'References',
           30: 'Co-citing',
           31: 'Language',
           32: 'RecordID',
           ## dataplus, evs
           33: 'Operationalisations'
       }
       return order

    def _setGenre(self):
        genre = self.xpath(self._metadata, '//mods:mods/mods:genre/text()')
        if genre and '/' in genre: genre = genre.split('/')[-1]
####### The following will become redundant because in the future there will be
####### no journals with genre == 'series'
        #if genre == 'series' and self._recordId.split(':')[0] in ['opc-uvt-nl']:
        #    journal = 0
        #    locations = nodeset(self._metadata, '//mods:mods/mods:location/mods:holdingSimple/mods:copyInformation/mods:shelfLocator')
        #    for loc in locations:
        #        t = self.xpath(loc, 'text()').split()[0]
        #        if t in ['T', 'E', 'TF']: journal = 1
        #    if journal: genre = "journal"
        self._genre = genre

    def getMods(self, recordId):
        # FIXME tijdelijke oplossing
        baseurl = self.serverConfig.get('url', section='dbserver')
        self.logger.debug('============= baseurl: ' + baseurl)
        if self._recordId != recordId:
            self.logger.debug("getMods: sru search")
            # FIXME niet overal nog ondersteuning van recordSchema mods
            url = baseurl + \
                "/sru?version=1.1&operation=searchRetrieve&recordSchema=mods&query=meta.upload.id+exact+\"" + \
                recordId + "\""
            self.logger.debug('url: ' + url )
            response = urlopen(url)
            self.logger.debug(self.infra.prpt('response', response))
            _metadata = parse(response)
        else: _metadata = self._metadata
        # FIXME niet overal nog ondersteuning van recordSchema mods
#        self.logger.debug(tostring( _metadata))
        return self.xpath(_metadata, '//mods:mods')

    def _addName(self, names, name):
        family = ""
        given  = ""
        infix  = ""
        for namePart in self.nodeset(name, 'mods:namePart'):
            type = self.xpath(namePart, '@type')
            if type == 'family' or not type:
                n = self.xpath(namePart, 'text()')
                f = n.split(',', 1)
                # Hippo, van
                if len(f) == 2:
                    family = f[0]
                    if type: infix = f[1]
                    else: given = f[1]
                # van Hippo
                # no comma
                else:
                    n = n.replace('  ', ' ')
                    if type: family = n
                    else:
                        f = n.split(' ')
                        family = f[-1]
                        if not type: given = ' '.join(f[0:-1])
            elif type == 'given':  given  = self.xpath(namePart, 'text()')
        if family: names += family
        if given:
            names  += ', ' + given
            if infix: names += ' ' + infix
        if family: names += '; '
        return names

    def _citec(self):
        res = {}
        if self._collection == "repec":
            parts = self._recordId.split(':', 5)
            url = "ftp://ftp.repec.org/opt/amf/CitEc/" +  parts[3] + "/" + parts[4] + "/" + parts[5] + ".amf.xml"
            self.logger.debug("citec url: " + url)
            try:
                amf = parse(urlopen(url))
            except Exception:
                return {}
            refs = []
            references = amf.xpath("/amf:amf/amf:text/amf:references", namespaces=self.nsMap)
            if len(references) > 0:
                for r in references:
                    refid = r.xpath("amf:text/@ref", namespaces=self.nsMap)[0]
                    s = refid.split(":")
                    recId = "RePEc-"+s[1]+"-"+s[2]+":oai:"+refid
                    record = self.any.createRecord(recId, format = "short")
                    if not record:
                        recId = ""
                        refstring = r.xpath("acis:referencestring", namespaces=self.nsMap)[0].text
                        record = {"Citation": refstring}
                    refs.append((recId, record))
            references = amf.xpath("/amf:amf/amf:text/amf:reference", namespaces=self.nsMap)
            if len(references) > 0:
                for r in references:
                    string = r.xpath("amf:literal", namespaces=self.nsMap)[0].text
                    record = {"Citation": string}
                    refs.append(("", record))
            citedBy = []
            citedBys = amf.xpath("/amf:amf/amf:text/amf:isreferencedby", namespaces=self.nsMap)
            if len(citedBys) > 0:
                for c in citedBys:
                    by = c.xpath("amf:text/@ref", namespaces=self.nsMap)[0]
                    #RePEc:dgr:kubcen:200847
                    #RePEc-dgr-kubcen:oai:RePEc:dgr:kubcen:200847
                    s = by.split(":")
                    recId = "RePEc-"+s[1]+"-"+s[2]+":oai:"+by
                    record = self.any.createRecord(recId, format = "short")
                    citedBy.append((recId, record))
            if citedBy: res['Cited by']  = citedBy
            if refs:
                res['References'] = refs
                res['Co-citing'] = "http://citec.repec.org/cgi-bin/get_data.pl?o=all&h=repec:%s:%s:%s#cociting" %(parts[3], parts[4], parts[5])
        return res

    def _ddi3(self):
        ddi3 = {}
        if self._parts:
            for part in self._parts:
                if part.text == "ddi3":
                    dd = parse(self.any.getStream(self._recordId, 'ddi3'))
                    concepts = self.nodeset(dd, '/ddi3:DDIInstance/s:StudyUnit/l:LogicalProduct/l:VariableScheme/l:VariableGroup')
                    if concepts:
                        conceptDict = {}
                        for concept in concepts:
                            name = self.xpath(concept, 'r:Label/text()')
                            variables = []
                            conceptDict[name] = {}
                            variableURNs = self.nodeset(concept, 'l:VariableReference/r:URN')
                            if variableURNs:
                                for URN in variableURNs:
                                    path = '/ddi3:DDIInstance/s:StudyUnit/l:LogicalProduct/l:VariableScheme/l:Variable[@urn="%s"]' %URN.text
                                    var = self.nodeset(dd, path)[0]
                                    varLabel = self.xpath(var, 'r:Label/text()')
                                    varDescription = self.xpath(var, 'r:Description/text()')
                                    variables.append((varLabel, varDescription))
                            conceptDict[name]['variables'] = variables
                            syntax = self.xpath(concept, 'r:Description/text()')
                            if syntax: conceptDict[name]['syntax'] = syntax
                        ddi3['Concepts'] = conceptDict
                    syntax = self.xpath(dd, "/ddi3:DDIInstance/s:StudyUnit/r:Note/r:Content/text()")
                    if syntax: ddi3['Syntax'] = syntax
        return ddi3

    def _getDataset(self):
        if self._parts:
            for part in self._parts:
                if part.text == "ddi":
                    ddi = parse(self.any.getStream(self._recordId, 'ddi'))
                    url = ddi.xpath("//ddi:codeBook/ddi:docDscr/ddi:citation/ddi:holdings/@URI", namespaces=self.nsMap)[0]
                    return url
        return ''

    def _getDate(self):
        date = self.xpath(self._metadata, '//mods:originInfo/mods:dateIssued/text()')
        if date:
            altDate = date.split('(', 1)
            if len(altDate) == 2: date = altDate[1].rstrip(')')
            return date.split('-', 1)[0]

    def _getExtent(self):
        return self.xpath(self._metadata, '//mods:mods/mods:physicalDescription/mods:extent/text()')

    def _getKeywords(self):
        keywords = ""
        for topic in self.nodeset(self._metadata, '//mods:mods/mods:subject/mods:topic'):
            keywords += self.xpath(topic, 'text()') + '; '
        if keywords: return keywords.rstrip('; ')
        else: return ''

    def _getLanguage(self):
        return self.xpath(self._metadata, '//mods:mods/mods:language/mods:languageTerm/text()')

    def _getLocation(self):
        locations = self.nodeset(self._metadata, '//mods:mods/mods:location/mods:holdingSimple/mods:copyInformation/mods:shelfLocator')
        if locations:
            parts = self._recordId.split(':')
            s = []
            for loc in locations:
                titles = []
                # varianten:
                # u"Zie afzonderl\u0133ke publikaties"
                # "Zie afzonderlijke publicaties"
                # "zie afzonderlijke publicaties"
                # "zie afzonderlijke publ."
                #if loc.text.find("ie afzonderl") > 0:
                signatuur =  self.xpath(loc, "./text()")
                if signatuur.find("ie afzonderl") > 0:
                    query = "mods.relatedItem.recordInfo.recordIdentifier=" + parts[1]
                    stop = 100
                    total, recordIds = self._search(query, stop)
                    if total > 0 and total < stop + 1:
                        for recordId in recordIds:
                            record = self.any.createRecord(recordId, format='short')
                            url = "/id/" + recordId
                            titles.append((url, record['Title']))
                    elif total > stop:
                        url = "?query=%s+%s" %(parts[0], parts[1])
                        titles.append((url, "Display the separate parts"))
                    else:
                        titles.append(("", "No publications found"))
                noteset = self.nodeset(loc, "../mods:note")
                notes = []
                for note in noteset:
                    if not note.get("type"): notes.append(note.text)
                s.append((signatuur, titles, notes))
            return s

    def _search(self, query, stop):
        parsetree = parseCQL(query)
        return self.any.executeCQL(cqlAbstractSyntaxTree=parsetree, start=0, stop=stop)
        # sortBy=sortBy, sortDescending=sortDescending 

    def _getPersonAsSubject(self):
        names = ""
        for name in self.nodeset(self._metadata, '//mods:mods/mods:subject/mods:name/mods:namePart'):
            names += name.text + '; '
        if names: return names.rstrip('; ')
        else: return ''

    def _getRelatedTitles(self):
        pass
        #parts = self._recordId.split(':')
        #if self._genre == 'series':
        #    if parts and parts[0] == 'opc-uvt-nl':
        #        return '<a href="/?query=%s+%s ">Display the items in the series</a>' %(parts[0], parts[1])
        #else:
        #    host = self.xpath(self._metadata, '//mods:mods/mods:relatedItem[@type="host"]/mods:recordInfo/mods:recordIdentifier/text()')
        #    if host:
        #        return '<a href="/?query=%s+%s ">Display related publications</a>' %(parts[0], host)

    def _getSeries(self):
        out = ""
        series = (self.nodeset(self._metadata, '//mods:relatedItem[@type="series"]'))
        if series:
            for aseries in series:
                title = escape(self.xpath(aseries, 'mods:titleInfo/mods:title/text()'))
                if not title: continue
                ppn   = self.xpath(aseries, 'mods:recordInfo/mods:recordIdentifier/text()')
                part  = self.xpath(aseries, 'mods:part/mods:detail/mods:number/text()')
                if ppn:
                    out += '<a target="_blank" href="/?query=%s+%s">%s</a>' %(self._recordId.split(':')[0], ppn, title)
                else:
                    out += title
                if part: out += '<i>: ' + escape(part) + '</i>'
                out += '; '
            return out.rstrip('; ')

    def _getSource(self):
        editors = escape(self._getHostEditors())
        title = escape(self._getHostTitle())
        if not title: return ''
        if editors: return editors + ' ' + '<i>' + title + '</i>'
        date = self._getDate()
        volume    = self.xpath(self._metadata, '//mods:part/mods:detail[@type="volume"]/mods:number/text()')
        issue     = self.xpath(self._metadata, '//mods:part/mods:detail[@type="issue"]/mods:number/text()')
        firstpage = self.xpath(self._metadata, '//mods:part/mods:extent/mods:start/text()')
        endpage   = self.xpath(self._metadata, '//mods:part/mods:extent/mods:end/text()')
        if volume: title += ' vol. ' + volume
        if date: title += ' (' + date + ')'
        if issue: title += ' nr. ' + issue
        if firstpage:
            title += ' p.' + firstpage
            if endpage: title += '-' + endpage
        part = self.xpath(self._metadata, '//mods:relatedItem[@type="host"]/mods:part/mods:text/text()')
        if part: title += '. ' + part
        return title

    def _getTitle(self):
        title = self.xpath(self._metadata, '//mods:mods/mods:titleInfo/mods:title/text()')
        if not title:
            title = self.xpath(self._metadata, '//mods:relatedItem[@type="host"]/mods:titleInfo/mods:title/text()')
            if title: title = '[' + title + ']'
            #fixme: naar config
            else: title = '[NO TITLE AVAILABLE]'
        subTitles = self.nodeset(self._metadata, '//mods:mods/mods:titleInfo/mods:subTitle/text()')
        if subTitles:
            for subTitle in subTitles:
                title += ": " + subTitle
        return escape(title)

    def _handleNames(self):
        authors = ""
        collaborators = ""
        editors = ""
        supervisors = ""
        corporation = ""
        modsNames = (self.nodeset(self._metadata, '//mods:mods/mods:name'))
        if not modsNames: return {}
        for name in modsNames:
            role = self.xpath(name, 'mods:role/mods:roleTerm/text()')
            if   role == 'aut' or not role:
                authors       = self._addName(authors,       name)
            elif role == 'clb':
                collaborators = self._addName(collaborators, name)
            elif role == 'edt':
                editors       = self._addName(editors,       name)
            elif role == 'ths':
                supervisors   = self._addName(supervisors,   name)
            elif role == 'dgg':
                content = 'Degree granted by ' + str(self.xpath(name, 'mods:namePart/text()'))
                corporation   = content
        names = {}
        if corporation:
            names['Corporation'] = corporation
        if authors:
            names['Author(s)'] = authors.rstrip(' ;')
        if collaborators:
            names['Collaborator(s)'] = collaborators.rstrip(' ;')
        if editors:
            names['Editor(s)'] = editors.rstrip(' ;')
        if supervisors:
            names['Supervisor(s)'] = supervisors.rstrip(' ;')
        return names

    def _getHostTitle(self):
        return self.xpath(self._metadata, '//mods:mods/mods:relatedItem[@type="host"]/mods:titleInfo/mods:title/text()')

    def _getHostEditors(self):
        editors = ""
        family = ""
        given = ""
        for name in self.nodeset(self._metadata, '//mods:mods/mods:relatedItem/mods:name'):
            if self.xpath(name, 'mods:role/mods:roleTerm/text()') == 'edt':
                for namePart in self.nodeset(name, 'mods:namePart'):
                    type = self.xpath(namePart, '@type')
                    if type == 'family':  family = self.xpath(namePart, 'text()')
                    elif type == 'given':  given = self.xpath(namePart, 'text()')
                if family: editors += family
                if given:  editors += ', ' + given
                if family: editors += '; '
                family = ""
                given = ""
        if editors: return editors.rstrip('; ') + ' (Ed.)'
        else: return ''

    def _getAbstract(self):
        return escape(self.xpath(self._metadata, '//mods:abstract/text()'))

    def _getUrls(self):
        urls = []
        url = self.xpath(self._metadata, '//mods:location/mods:url/text()')
        if url: urls.append(url)
        url = self.xpath(self._metadata, '//mods:location/mods:holdingSimple/mods:copyInformation/mods:electronicLocator/text()')
        if url: urls.append(url)
        for Item in self.nodeset(self._metadata, '//didl:DIDL/didl:Item/didl:Item'):
            if self.xpath(Item, 'didl:Descriptor/didl:Statement/rdf:type/text()')        == 'info:eu-repo/semantics/objectFile' or \
               self.xpath(Item, 'didl:Descriptor/didl:Statement/dip:ObjectType/text()')  == 'info:eu-repo/semantics/objectFile' or \
               self.xpath(Item, 'didl:Descriptor/didl:Statement/rdf:type/@rdf:resource') == 'info:eu-repo/semantics/objectFile':
                url = self.xpath(Item, 'didl:Component/didl:Resource/@ref')
                if url: urls.append(url)
        return urls

    def _getNotes(self):
        notes = ""
        count = 0
        for note in self.nodeset(self._metadata, '//mods:mods/mods:note/text()'):
             count += 1
             if count > 1:
                 notes += ". "
             notes += escape(note)
        return notes

    def _getReference(self):
        return self.xpath(self._metadata, '//mods:mods/mods:relatedItem[@type="isReferencedBy"]/mods:titleInfo/mods:title/text()')

    def _getPublisher(self):
        place = self.xpath(self._metadata, '//mods:mods/mods:originInfo/mods:place/mods:placeTerm/text()')
        name = self.xpath(self._metadata, '//mods:mods/mods:originInfo/mods:publisher/text()')
        publisher = ""
        if name:
           publisher = ""
           if place: publisher = place + ": "
           publisher += name
        return escape(publisher)

    def _getClassification(self):
        string = ""
        for classification in self.nodeset(self._metadata, '//mods:mods/mods:classification'):
            string += self.xpath(classification, 'text()') + '; '
        return string.rstrip('; ')

    def _getGeneratedJel(self):
        if self._parts:
            for part in self._parts:
                if part.text == "jel":
                    mods = parse(self.any.getStream(self._recordId, 'jel'))
                    string = ""
                    for classification in self.nodeset(mods, '//mods:mods/mods:classification'):
                        string += self.xpath(classification, 'text()') + '; '
                    return string.rstrip('; ')
        return ''

    def _handleIdentifiers(self):
        identifiers = (self.nodeset(self._metadata, '//mods:identifier'))
        res = {}

        if not identifiers: return res
        issns = ""
        isbns = ""
        doi   = ""
        for id in identifiers:
            idParts = self.xpath(id, 'text()').split(':')
            ### urn:{issn|issbn}:<value>
            ### info:{doi}/<value>
            if idParts[0] == 'urn':
                if len(idParts) < 3: break
                if idParts[1] == 'issn' or idParts[1] == 'ISSN':
                    if issns: issns += "; "
                    issns += idParts[2]
                elif idParts[1] == 'isbn' or idParts[1] == 'ISBN':
                    if isbns: isbns += "; "
                    displayLabel = self.xpath(id, '@displayLabel')
                    if displayLabel: isbns += displayLabel
                    else: isbns += idParts[2]
            elif idParts[0] == 'info':
                parts = idParts[1].split('/', 1)
                if parts[0] == 'doi': doi = parts[1]

        if issns: res['ISSN'] = issns
        if isbns: res['ISBN'] = isbns
        if doi  : res['Doi']  = doi
        return res

    def makeDictionary(self):
        self.makeShortDict()
        add(self.record, 'Language', self._getLanguage())
        add(self.record, 'Dataset', self._getDataset())
        add(self.record, 'Abstract', self._getAbstract())
        add(self.record, 'Person as subject', self._getPersonAsSubject())
        add(self.record, 'Keyword(s)', self._getKeywords())
        add(self.record, 'Classification', self._getClassification())
        add(self.record, 'Generated JEL', self._getGeneratedJel())
        add(self.record, 'Publisher', self._getPublisher())
        add(self.record, 'Extent', self._getExtent())
        add(self.record, 'Series', self._getSeries())
        add(self.record, 'Note(s)', self._getNotes())
        #add(self.record, 'Reference', self._getReference())
        add(self.record, 'Location', self._getLocation())
        add(self.record, 'Related Titles', self._getRelatedTitles())
        add(self.record, 'RecordID', self._recordId)
        #self.record.update(self._citec())
        add(self.record, 'Operationalisations', self._ddi3())

    def makeShortDict(self):
        self.record = {}
        add(self.record, 'Title', self._getTitle())
        self.record.update(self._handleNames())
        add(self.record, 'Source', self._getSource())
        add(self.record, 'Publication date', self._getDate())
        self.record.update(self._handleIdentifiers())
        add(self.record, 'Publication type', self._genre)
        add(self.record, 'URL(s)', self._getUrls())

    def makeCitecDict(self):
        self.record.update(self._citec())

class RecordXml(Observable):
    def __init__(self):
         Observable.__init__(self)

    def handleRequest(self, RequestURI='', **kwargs):
####### TP: toevoegen het afvangen van exceptions: 1. er is geen record id,
####### 2. er wordt geen metadata gevonden
        Scheme, Netloc, Path, Query, Fragment = urlsplit(RequestURI)
        arguments = parse_qs(Query)
        recordId = arguments.get('recordId', [''])[0]
        try:
            self.any.checkRecordAccess(recordId, kwargs)
        except:
            yield forbiddenXml()
            yield """<?xml version="1.0" encoding="UTF-8"?>
<denied/>"""
            return
        record = self.any.createRecord(recordId)
        yield httputils.okXml
        yield """<?xml version="1.0" encoding="UTF-8"?>
<record>"""

####### TP: labels zijn in het Engels. Toevoegen: ondersteunen van andere talen.
####### TP: ook handig is om per field het type toe te voegen (taalonafhankelijk)
        order = self.any.getOrder()
        order.keys().sort()
        for key in order.keys():
            label = order[key]
            if label in record:
                if label == 'URL(s)':
                    for url in record['URL(s)']:
                        yield '\n <field><label>URL</label><content>%s</content></field>' %(makeUrlContent(url))
                elif label == 'Doi':
                        yield '\n <field><label>Doi</label><content>%s</content></field>' %(makeDoiContent(record['Doi']))
                elif label == 'Dataset':
                        yield '\n <field><label>Dataset</label><content>%s</content></field>' %(makeUrlContent(record['Dataset']))
                elif label in ['Related Titles', 'Series', 'Source']:
                    yield '\n <field><label>%s</label><content>%s</content></field>' %(label, record[label])
                elif label in ['Location']:
                    yield '\n <field><label>%s</label><content>' %(label)
                    for item in record[label]:
                        if item is not record[label][0]:
                            yield "<br/><br/>"
                        yield item[0]
                        for note in item[2]:
                            yield  "<br/>" + note        
                        if item[1]:
                            titles = item[1]
                            yield '\n  <ul>'
                            for title in titles:
                                if title[0]:
                                    yield '\n   <li><a target="_blank" href="%s">%s</a></li>' %(title[0], escape(title[1]))
                                else:
                                    yield '\n   %s' %escape(title[1])
                            yield '\n  </ul>'
                    yield '</content></field>'
                elif label in ['Cited by', 'References']:
                    yield '\n <field><label>%s</label><content><ul>' %(label)
                    for item in record[label]:
                        if item[0] and item[1]:
                            rec = item[1]
                            citation =  '<li>'
                            if 'Author(s)' in rec: citation += rec['Author(s)']
                            if 'Publication date' in rec:
                                citation += ' (' + \
                                            rec['Publication date'] + \
                                            ')'
                            citation += ' <a target="_blank" href="?query=' + \
                                        item[0] + \
                                        '&amp;displayType=single">' + \
                                        escape(rec['Title']) + \
                                        '</a> '
                            if 'Source' in rec: citation += rec['Source']
                            citation += '</li>'
                            yield '\n %s' %(citation)
                        elif item[1]:
                            yield '\n <li>%s</li>' %(escape(item[1]['Citation']))
                    yield '\n </ul></content></field>'
                elif label in ['Co-citing']:
                    yield '\n <field><label>Co-citing</label><content><a target="_blank" href="%s">Documents</a> which have cited the same bibliography</content></field>' %escape(record[label])

                elif label in ['RecordID']:
                    yield '\n <field><label>RecordID</label><content>%s</content></field>' %(makePermaLink(recordId))

                elif label == "Operationalisations":
                    for concept in record[label]["Concepts"]:
                        yield '\n <field><label>%s</label><content>' %concept
                        yield '<ul>'
                        for var in record[label]["Concepts"][concept]["variables"]:
                            yield '<li>' + var[0] + ": " + var[1] + "</li>"
                        yield '</ul>'
                        if "syntax" in record[label]["Concepts"][concept]:
                            for line in record[label]["Concepts"][concept]["syntax"].split('\n'):
                                yield escape(line) + "<br/>"
                        yield '</content></field>'
                    if "Syntax" in record[label]:
                        yield '\n <field><label>Syntax</label><content>'
                        for line in record[label]["Syntax"].split('\n'):
                            yield escape(line) + "<br/>"
                        yield '</content></field>'

                else:
                    yield '\n <field><label>%s</label><content>%s</content></field>' %(label, escape(record[label]))
        yield '\n</record>'

class Citec(Observable):
    def __init__(self):
         Observable.__init__(self)

    def handleRequest(self, RequestURI='', **kwargs):
####### TP: toevoegen het afvangen van exceptions: 1. er is geen record id,
####### 2. er wordt geen metadata gevonden
        Scheme, Netloc, Path, Query, Fragment = urlsplit(RequestURI)
        arguments = parse_qs(Query)
        recordId = arguments.get('recordId', [''])[0]
        record = self.any.createRecord(recordId, format="citec")
        yield httputils.okXml
        yield """<?xml version="1.0" encoding="UTF-8"?>
<record>"""
        for label in record.keys():
            if label in ['Cited by', 'References']:
                yield '\n <field><label>%s</label><content><ul>' %(label)
                for item in record[label]:
                    if item[0] and item[1]:
                        rec = item[1]
                        citation =  '<li>'
                        if 'Author(s)' in rec: citation += rec['Author(s)']
                        if 'Publication date' in rec:
                            citation += ' (' + \
                                        rec['Publication date'] + \
                                        ')'
                        citation += ' <a target="_blank" href="?query=' + \
                                    item[0] + \
                                    '&amp;displayType=single">' + \
                                    escape(rec['Title']) + \
                                    '</a> '
                        if 'Source' in rec: citation += rec['Source']
                        citation += '</li>'
                        yield '\n %s' %(citation)
                    elif item[1]:
                        yield '\n <li>%s</li>' %(escape(item[1]['Citation']))
                yield '\n </ul></content></field>'
            elif label in ['Co-citing']:
                yield '\n <field><label>Co-citing</label><content><a target="_blank" href="%s">Documents</a> which have cited the same bibliography</content></field>' %escape(record[label])
        yield '\n</record>'

class RecordJSON(Observable):
    def __init__(self):
         Observable.__init__(self)

    def handleRequest(self, RequestURI='', **kwargs):
####### TP: toevoegen het afvangen van exceptions: 1. er is geen record id,
####### 2. er wordt geen metadata gevonden
        Scheme, Netloc, Path, Query, Fragment = urlsplit(RequestURI)
        arguments = parse_qs(Query)
        recordId = arguments.get('recordId', [''])[0]
        record = self.any.createRecord(recordId)
        yield okJSON()
        yield JSONEncoder().encode(record)
